@@ -2,7 +2,7 @@
 
 require "test_helper"
 
-class WorkerTest < ActiveSupport::TestCase
+class WorkerTest < ActionCable::TestCase
   class Receiver
     attr_accessor :last_action
 
