@@ -1,9 +1,11 @@
-require 'abstract_unit'
-require 'action_controller/metal/strong_parameters'
+# frozen_string_literal: true
+
+require "abstract_unit"
+require "action_controller/metal/strong_parameters"
 
 class MultiParameterAttributesTest < ActiveSupport::TestCase
   test "permitted multi-parameter attribute keys" do
-    params = ActionController::Parameters.new({
+    params = ActionController::Parameters.new(
       book: {
         "shipped_at(1i)"   => "2012",
         "shipped_at(2i)"   => "3",
@@ -15,12 +17,11 @@ class MultiParameterAttributesTest < ActiveSupport::TestCase
         "published_at(3i)" => "5",
         "price(1)"         => "R$",
         "price(2f)"        => "2.02"
-      }
-    })
+      })
 
     permitted = params.permit book: [ :shipped_at, :price ]
 
-    assert permitted.permitted?
+    assert_predicate permitted, :permitted?
 
     assert_equal "2012", permitted[:book]["shipped_at(1i)"]
     assert_equal "3", permitted[:book]["shipped_at(2i)"]
