@@ -1,6 +1,10 @@
 class MessagesController < ActionController::Base
   before_action :set_message, only: [:show, :edit, :update, :destroy]
 
+  # This class intentionally does not extend ApplicationController, so the
+  # layout must be set manually. See commit 614e813 for details
+  layout "application"
+
   # GET /messages
   def index
     @messages = Message.all
@@ -26,7 +30,7 @@ class MessagesController < ActionController::Base
     if @message.save
       redirect_to @message, notice: 'Message was successfully created.'
     else
-      render :new
+      render :new, status: :unprocessable_entity
     end
   end
 
@@ -35,7 +39,7 @@ class MessagesController < ActionController::Base
     if @message.update(message_params)
       redirect_to @message, notice: 'Message was successfully updated.'
     else
-      render :edit
+      render :edit, status: :unprocessable_entity
     end
   end
 
