@@ -15,7 +15,7 @@ module Arel
         next if Nodes::SqlLiteral == klass
         next if Nodes::BindParam == klass
         next if /^Arel::Nodes::(?:Test|.*Test$)/.match?(klass.name)
-        assert klass.ancestors.include?(Nodes::Node), klass.name
+        assert_includes klass.ancestors, Nodes::Node
       end
     end
   end
