@@ -2,7 +2,7 @@
 
 require "date"
 
-class DateTime #:nodoc:
+class DateTime # :nodoc:
   # No DateTime is ever blank:
   #
   #   DateTime.now.blank? # => false
@@ -11,4 +11,8 @@ class DateTime #:nodoc:
   def blank?
     false
   end
+
+  def present?
+    true
+  end
 end
