@@ -7,7 +7,12 @@ class ChannelGeneratorTest < Rails::Generators::TestCase
   include GeneratorsTestHelper
   tests Rails::Generators::ChannelGenerator
 
-  def test_application_cable_skeleton_is_created
+  setup do
+    use_with_javascript
+    use_under_importmap
+  end
+
+  test "shared channel files are created" do
     run_generator ["books"]
 
     assert_file "app/channels/application_cable/channel.rb" do |cable|
@@ -19,7 +24,7 @@ class ChannelGeneratorTest < Rails::Generators::TestCase
     end
   end
 
-  def test_channel_is_created
+  test "specific channel files are created under importmap" do
     run_generator ["chat"]
 
     assert_file "app/channels/chat_channel.rb" do |channel|
@@ -27,11 +32,22 @@ class ChannelGeneratorTest < Rails::Generators::TestCase
     end
 
     assert_file "app/javascript/channels/chat_channel.js" do |channel|
-      assert_match(/import consumer from "\.\/consumer"\s+consumer\.subscriptions\.create\("ChatChannel/, channel)
+      assert_match(/import consumer from "channels\/consumer"\s+consumer\.subscriptions\.create\("ChatChannel/, channel)
     end
   end
 
-  def test_channel_with_multiple_actions_is_created
+  test "specific channel files are created under node" do
+    use_under_node
+    generator(["chat"]).stub(:install_javascript_dependencies, true) do
+      run_generator_instance
+
+      assert_file "app/javascript/channels/chat_channel.js" do |channel|
+        assert_match(/import consumer from ".\/consumer"\s+consumer\.subscriptions\.create\("ChatChannel/, channel)
+      end
+    end
+  end
+
+  test "channel with multiple actions is created" do
     run_generator ["chat", "speak", "mute"]
 
     assert_file "app/channels/chat_channel.rb" do |channel|
@@ -41,13 +57,57 @@ class ChannelGeneratorTest < Rails::Generators::TestCase
     end
 
     assert_file "app/javascript/channels/chat_channel.js" do |channel|
-      assert_match(/import consumer from "\.\/consumer"\s+consumer\.subscriptions\.create\("ChatChannel/, channel)
+      assert_match(/import consumer from "channels\/consumer"\s+consumer\.subscriptions\.create\("ChatChannel/, channel)
       assert_match(/,\n\n  speak/, channel)
       assert_match(/,\n\n  mute: function\(\) \{\n    return this\.perform\('mute'\);\n  \}\n\}\);/, channel)
     end
   end
 
-  def test_channel_asset_is_not_created_when_skip_assets_is_passed
+  test "shared channel javascript files are created" do
+    run_generator ["books"]
+
+    assert_file "app/javascript/channels/index.js"
+    assert_file "app/javascript/channels/consumer.js"
+  end
+
+  test "import channels in javascript entrypoint" do
+    run_generator ["books"]
+
+    assert_file "app/javascript/application.js" do |entrypoint|
+      assert_match %r|import "channels"|, entrypoint
+    end
+  end
+
+  test "import channels in javascript entrypoint under node" do
+    use_under_node
+    generator(["chat"]).stub(:install_javascript_dependencies, true) do
+      run_generator_instance
+
+      assert_file "app/javascript/application.js" do |entrypoint|
+        assert_match %r|import "./channels"|, entrypoint
+      end
+    end
+  end
+
+  test "pin javascript dependencies" do
+    run_generator ["chat"]
+
+    assert_file "config/importmap.rb" do |content|
+      assert_match %r|pin "@rails/actioncable"|, content
+      assert_match %r|pin_all_from "app/javascript/channels"|, content
+    end
+  end
+
+  test "first setup only happens once" do
+    run_generator ["chat"]
+    assert_file "app/javascript/channels/consumer.js"
+
+    FileUtils.rm("#{destination_root}/app/javascript/channels/consumer.js")
+    run_generator ["another"]
+    assert_no_file "app/javascript/channels/consumer.js"
+  end
+
+  test "javascripts not generated when assets are skipped" do
     run_generator ["chat", "--skip-assets"]
 
     assert_file "app/channels/chat_channel.rb" do |channel|
@@ -57,17 +117,7 @@ class ChannelGeneratorTest < Rails::Generators::TestCase
     assert_no_file "app/javascript/channels/chat_channel.js"
   end
 
-  def test_consumer_js_is_created_if_not_present_already
-    run_generator ["chat"]
-    FileUtils.rm("#{destination_root}/app/javascript/channels/index.js")
-    FileUtils.rm("#{destination_root}/app/javascript/channels/consumer.js")
-    run_generator ["camp"]
-
-    assert_file "app/javascript/channels/index.js"
-    assert_file "app/javascript/channels/consumer.js"
-  end
-
-  def test_invokes_default_test_framework
+  test "invokes default test framework" do
     run_generator %w(chat -t=test_unit)
 
     assert_file "test/channels/chat_channel_test.rb" do |test|
@@ -77,7 +127,7 @@ class ChannelGeneratorTest < Rails::Generators::TestCase
     end
   end
 
-  def test_channel_on_revoke
+  test "revoking" do
     run_generator ["chat"]
     run_generator ["chat"], behavior: :revoke
 
@@ -87,11 +137,10 @@ class ChannelGeneratorTest < Rails::Generators::TestCase
 
     assert_file "app/channels/application_cable/channel.rb"
     assert_file "app/channels/application_cable/connection.rb"
-    assert_file "app/javascript/channels/index.js"
     assert_file "app/javascript/channels/consumer.js"
   end
 
-  def test_channel_suffix_is_not_duplicated
+  test "channel suffix is not duplicated" do
     run_generator ["chat_channel"]
 
     assert_no_file "app/channels/chat_channel_channel.rb"
@@ -103,4 +152,21 @@ class ChannelGeneratorTest < Rails::Generators::TestCase
     assert_no_file "test/channels/chat_channel_channel_test.rb"
     assert_file "test/channels/chat_channel_test.rb"
   end
+
+  private
+    def use_with_javascript
+      FileUtils.mkdir_p("#{destination_root}/app/javascript")
+      FileUtils.touch("#{destination_root}/app/javascript/application.js")
+    end
+
+    def use_under_importmap
+      FileUtils.mkdir_p("#{destination_root}/config")
+      FileUtils.touch("#{destination_root}/config/importmap.rb")
+      FileUtils.rm_rf("#{destination_root}/package.json")
+    end
+
+    def use_under_node
+      FileUtils.touch("#{destination_root}/package.json")
+      FileUtils.rm_rf("#{destination_root}/config/importmap.rb")
+    end
 end
