@@ -1,29 +1,85 @@
 # frozen_string_literal: true
 
+require "plugin_helpers"
 require "generators/generators_test_helper"
 require "rails/generators/rails/plugin/plugin_generator"
 require "generators/shared_generator_tests"
 require "rails/engine/updater"
 
 DEFAULT_PLUGIN_FILES = %w(
+  .git
+  .github/dependabot.yml
+  .github/workflows/ci.yml
   .gitignore
+  .rubocop.yml
   Gemfile
-  Rakefile
-  README.md
-  bukkits.gemspec
   MIT-LICENSE
-  lib
+  README.md
+  Rakefile
+  bin/rubocop
+  bin/test
+  bukkits.gemspec
   lib/bukkits.rb
-  lib/tasks/bukkits_tasks.rake
+  lib/bukkits/railtie.rb
   lib/bukkits/version.rb
+  lib/tasks/bukkits_tasks.rake
   test/bukkits_test.rb
+  test/dummy/Rakefile
+  test/dummy/app/assets/images/.keep
+  test/dummy/app/assets/stylesheets/application.css
+  test/dummy/app/channels/application_cable/channel.rb
+  test/dummy/app/channels/application_cable/connection.rb
+  test/dummy/app/controllers/application_controller.rb
+  test/dummy/app/controllers/concerns/.keep
+  test/dummy/app/helpers/application_helper.rb
+  test/dummy/app/jobs/application_job.rb
+  test/dummy/app/mailers/application_mailer.rb
+  test/dummy/app/models/application_record.rb
+  test/dummy/app/models/concerns/.keep
+  test/dummy/app/views/layouts/application.html.erb
+  test/dummy/app/views/layouts/mailer.html.erb
+  test/dummy/app/views/layouts/mailer.text.erb
+  test/dummy/app/views/pwa/manifest.json.erb
+  test/dummy/app/views/pwa/service-worker.js
+  test/dummy/bin/rails
+  test/dummy/bin/rake
+  test/dummy/bin/setup
+  test/dummy/config.ru
+  test/dummy/config/application.rb
+  test/dummy/config/boot.rb
+  test/dummy/config/cable.yml
+  test/dummy/config/database.yml
+  test/dummy/config/environment.rb
+  test/dummy/config/environments/development.rb
+  test/dummy/config/environments/production.rb
+  test/dummy/config/environments/test.rb
+  test/dummy/config/initializers/content_security_policy.rb
+  test/dummy/config/initializers/filter_parameter_logging.rb
+  test/dummy/config/initializers/inflections.rb
+  test/dummy/config/initializers/permissions_policy.rb
+  test/dummy/config/locales/en.yml
+  test/dummy/config/puma.rb
+  test/dummy/config/routes.rb
+  test/dummy/config/storage.yml
+  test/dummy/lib/assets/.keep
+  test/dummy/log/.keep
+  test/dummy/public/404.html
+  test/dummy/public/406-unsupported-browser.html
+  test/dummy/public/422.html
+  test/dummy/public/500.html
+  test/dummy/public/icon.png
+  test/dummy/public/icon.svg
+  test/dummy/storage/.keep
+  test/dummy/tmp/.keep
+  test/dummy/tmp/pids/.keep
+  test/dummy/tmp/storage/.keep
   test/test_helper.rb
-  test/dummy
 )
 
 class PluginGeneratorTest < Rails::Generators::TestCase
+  include PluginHelpers
   include GeneratorsTestHelper
-  destination File.join(Rails.root, "tmp/bukkits")
+  destination File.join(destination_root, "bukkits")
   arguments [destination_root]
 
   def application_path
@@ -66,12 +122,9 @@ class PluginGeneratorTest < Rails::Generators::TestCase
   def test_generating_without_options
     run_generator
     assert_file "README.md", /Bukkits/
-    assert_no_file "config/routes.rb"
-    assert_no_file "app/assets/config/bukkits_manifest.js"
     assert_file "test/test_helper.rb" do |content|
       assert_match(/require_relative.+test\/dummy\/config\/environment/, content)
       assert_match(/ActiveRecord::Migrator\.migrations_paths.+test\/dummy\/db\/migrate/, content)
-      assert_match(/Rails::TestUnitReporter\.executable = 'bin\/test'/, content)
     end
     assert_file "lib/bukkits/railtie.rb", /module Bukkits\n  class Railtie < ::Rails::Railtie\n  end\nend/
     assert_file "lib/bukkits.rb" do |content|
@@ -82,25 +135,47 @@ class PluginGeneratorTest < Rails::Generators::TestCase
       assert_match(/class BukkitsTest < ActiveSupport::TestCase/, content)
       assert_match(/assert Bukkits::VERSION/, content)
     end
-    assert_file "bin/test"
-    assert_no_file "bin/rails"
   end
 
-  def test_initializes_git_repo
+  def test_initializes_git_repo_with_main_branch_without_user_default
+    current_default_branch = `git config --global init.defaultBranch`
+    `git config --global --unset init.defaultBranch`
+
     run_generator
-    assert_directory ".git"
+    assert_file ".git/HEAD", /main/
+  ensure
+    if !current_default_branch.strip.empty?
+      `git config --global init.defaultBranch #{current_default_branch}`
+    end
+  end
+
+  def test_version_control_initializes_git_repo_with_user_default_branch
+    git_version = `git --version`[/\d+.\d+.\d+/]
+    return if Gem::Version.new(git_version) < Gem::Version.new("2.28.0")
+
+    current_default_branch = `git config --global init.defaultBranch`
+    `git config --global init.defaultBranch master`
+
+    run_generator
+    assert_file ".git/HEAD", /master/
+  ensure
+    if current_default_branch && current_default_branch.strip.empty?
+      `git config --global --unset init.defaultBranch`
+    elsif current_default_branch
+      `git config --global init.defaultBranch #{current_default_branch}`
+    end
   end
 
   def test_generating_in_full_mode_with_almost_of_all_skip_options
-    run_generator [destination_root, "--full", "-M", "-O", "-C", "-S", "-T", "--skip-active-storage"]
+    run_generator [destination_root, "--full", "-M", "-O", "-C", "-T", "--skip-active-storage", "--skip-active-job"]
     assert_file "bin/rails" do |content|
       assert_no_match(/\s+require\s+["']rails\/all["']/, content)
     end
+    assert_file "bin/rails", /#\s+require\s+["']active_job\/railtie["']/
     assert_file "bin/rails", /#\s+require\s+["']active_record\/railtie["']/
     assert_file "bin/rails", /#\s+require\s+["']active_storage\/engine["']/
     assert_file "bin/rails", /#\s+require\s+["']action_mailer\/railtie["']/
     assert_file "bin/rails", /#\s+require\s+["']action_cable\/engine["']/
-    assert_file "bin/rails", /#\s+require\s+["']sprockets\/railtie["']/
     assert_file "bin/rails", /#\s+require\s+["']rails\/test_unit\/railtie["']/
   end
 
@@ -111,19 +186,14 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     assert_file "test/integration/navigation_test.rb", /ActionDispatch::IntegrationTest/
   end
 
-  def test_inclusion_of_git_source
-    run_generator [destination_root]
-    assert_file "Gemfile", /git_source/
-  end
-
   def test_inclusion_of_a_debugger
     run_generator [destination_root, "--full"]
-    if defined?(JRUBY_VERSION) || RUBY_ENGINE == "rbx"
+    if defined?(JRUBY_VERSION)
       assert_file "Gemfile" do |content|
-        assert_no_match(/byebug/, content)
+        assert_no_match(/debug/, content)
       end
     else
-      assert_file "Gemfile", /# gem 'byebug'/
+      assert_file "Gemfile", /# gem "debug"/
     end
   end
 
@@ -146,19 +216,13 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     assert_file "bin/rails", /APP_PATH/
   end
 
-  def test_generating_adds_dummy_app_without_javascript_and_assets_deps
-    run_generator
-
-    assert_file "test/dummy/app/assets/stylesheets/application.css"
-  end
-
   def test_ensure_that_plugin_options_are_not_passed_to_app_generator
-    FileUtils.cd(Rails.root)
+    FileUtils.cd(fixtures_root)
     assert_no_match(/It works from file!.*It works_from_file/, run_generator([destination_root, "-m", "lib/template.rb"]))
   end
 
   def test_ensure_that_test_dummy_can_be_generated_from_a_template
-    FileUtils.cd(Rails.root)
+    FileUtils.cd(fixtures_root)
     run_generator([destination_root, "-m", "lib/create_test_dummy_template.rb", "--skip-test"])
     assert_directory "spec/dummy"
     assert_no_directory "test"
@@ -174,9 +238,7 @@ class PluginGeneratorTest < Rails::Generators::TestCase
   def test_default_database_dependency_is_sqlite
     run_generator
     assert_file "test/dummy/config/database.yml", /sqlite/
-    assert_file "Gemfile" do |contents|
-      assert_match_sqlite3(contents)
-    end
+    assert_gem "sqlite3"
   end
 
   def test_custom_database_dependency
@@ -204,31 +266,27 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     assert_file "test/dummy/config/database.yml", /postgres/
   end
 
-  def test_generation_runs_bundle_install
-    assert_generates_without_bundler
+  def test_no_duplicate_gemfile_entries_when_using_prerelease
+    run_generator_using_prerelease [destination_root, "--dev"]
+
+    assert_file "Gemfile" do |content|
+      assert_equal 1, content.scan(/gem "sqlite3"/).length
+    end
   end
 
-  def test_dev_option
-    assert_generates_without_bundler(dev: true)
-    rails_path = File.expand_path("../../..", Rails.root)
-    assert_file "Gemfile", /^gem\s+["']rails["'],\s+path:\s+["']#{Regexp.escape(rails_path)}["']$/
-  end
+  def test_generation_does_not_run_bundle_install
+    generator([destination_root])
+    run_generator_instance
 
-  def test_edge_option
-    assert_generates_without_bundler(edge: true)
-    assert_file "Gemfile", %r{^gem\s+["']rails["'],\s+github:\s+["']#{Regexp.escape("rails/rails")}["'],\s+branch:\s+["']#{Regexp.escape("6-1-stable")}["']$}
+    assert_empty @bundle_commands
   end
 
   def test_generation_does_not_run_bundle_install_with_full_and_mountable
-    assert_generates_without_bundler(mountable: true, full: true, dev: true)
-    assert_no_file "#{destination_root}/Gemfile.lock"
-  end
+    generator([destination_root], mountable: true, full: true)
+    run_generator_instance
 
-  def test_skip_javascript
-    run_generator [destination_root, "--skip-javascript", "--mountable"]
-    assert_file "app/views/layouts/bukkits/application.html.erb" do |content|
-      assert_no_match "javascript_pack_tag", content
-    end
+    assert_empty @bundle_commands
+    assert_no_file "#{destination_root}/Gemfile.lock"
   end
 
   def test_skip_action_mailer_and_skip_active_job_with_mountable
@@ -249,31 +307,15 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     assert_no_directory "app/jobs"
   end
 
-  def test_template_from_dir_pwd
-    FileUtils.cd(Rails.root)
-    assert_match(/It works from file!/, run_generator([destination_root, "-m", "lib/template.rb"]))
-  end
-
-  def test_ensure_that_tests_work
-    run_generator
-    FileUtils.cd destination_root
-    quietly { system "bundle install" }
-    assert_match(/1 runs, 1 assertions, 0 failures, 0 errors/, `bin/test 2>&1`)
-  end
-
-  def test_ensure_that_tests_works_in_full_mode
-    run_generator [destination_root, "--full", "--skip_active_record"]
-    FileUtils.cd destination_root
-    quietly { system "bundle install" }
-    assert_match(/1 runs, 1 assertions, 0 failures, 0 errors/, `bundle exec rake test 2>&1`)
-  end
-
   def test_ensure_that_migration_tasks_work_with_mountable_option
     run_generator [destination_root, "--mountable"]
-    FileUtils.cd destination_root
-    quietly { system "bundle install" }
-    output = `bin/rails db:migrate 2>&1`
-    assert $?.success?, "Command failed: #{output}"
+    prepare_plugin(destination_root)
+
+    in_plugin_context(destination_root) do
+      quietly { system "bundle install" }
+      output = `bin/rails db:migrate 2>&1`
+      assert_predicate $?, :success?, "Command failed: #{output}"
+    end
   end
 
   def test_creating_engine_in_full_mode
@@ -343,7 +385,7 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     assert_file "app/controllers/bukkits/application_controller.rb", /module Bukkits\n  class ApplicationController < ActionController::Base/
     assert_file "app/models/bukkits/application_record.rb", /module Bukkits\n  class ApplicationRecord < ActiveRecord::Base/
     assert_file "app/jobs/bukkits/application_job.rb", /module Bukkits\n  class ApplicationJob < ActiveJob::Base/
-    assert_file "app/mailers/bukkits/application_mailer.rb", /module Bukkits\n  class ApplicationMailer < ActionMailer::Base\n    default from: 'from@example\.com'\n    layout 'mailer'\n/
+    assert_file "app/mailers/bukkits/application_mailer.rb", /module Bukkits\n  class ApplicationMailer < ActionMailer::Base\n    default from: "from@example\.com"\n    layout "mailer"\n/
     assert_file "app/helpers/bukkits/application_helper.rb", /module Bukkits\n  module ApplicationHelper/
     assert_file "app/views/layouts/bukkits/application.html.erb" do |contents|
       assert_match "<title>Bukkits</title>", contents
@@ -356,8 +398,8 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     assert_file "test/test_helper.rb" do |content|
       assert_match(/ActiveRecord::Migrator\.migrations_paths.+\.\.\/test\/dummy\/db\/migrate/, content)
       assert_match(/ActiveRecord::Migrator\.migrations_paths.+<<.+\.\.\/db\/migrate/, content)
-      assert_match(/ActionDispatch::IntegrationTest\.fixture_path = ActiveSupport::TestCase\.fixture_pat/, content)
-      assert_no_match(/Rails::TestUnitReporter\.executable = 'bin\/test'/, content)
+      assert_match(/ActionDispatch::IntegrationTest\.fixture_paths = ActiveSupport::TestCase\.fixture_pat/, content)
+      assert_no_match(/Rails::TestUnitReporter\.executable = "bin\/test"/, content)
     end
     assert_no_file "bin/test"
   end
@@ -368,14 +410,14 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     assert_file "hyphenated-name/app/assets/stylesheets/hyphenated/name"
     assert_file "hyphenated-name/app/assets/images/hyphenated/name"
     assert_file "hyphenated-name/config/routes.rb",                                          /Hyphenated::Name::Engine\.routes\.draw do/
-    assert_file "hyphenated-name/lib/hyphenated/name/version.rb",                            /module Hyphenated\n  module Name\n    VERSION = '0\.1\.0'\n  end\nend/
+    assert_file "hyphenated-name/lib/hyphenated/name/version.rb",                            /module Hyphenated\n  module Name\n    VERSION = "0\.1\.0"\n  end\nend/
     assert_file "hyphenated-name/lib/hyphenated/name/engine.rb",                             /module Hyphenated\n  module Name\n    class Engine < ::Rails::Engine\n      isolate_namespace Hyphenated::Name\n    end\n  end\nend/
     assert_file "hyphenated-name/lib/hyphenated/name.rb",                                    /require "hyphenated\/name\/engine"/
     assert_file "hyphenated-name/test/dummy/config/routes.rb",                               /mount Hyphenated::Name::Engine => "\/hyphenated-name"/
     assert_file "hyphenated-name/app/controllers/hyphenated/name/application_controller.rb", /module Hyphenated\n  module Name\n    class ApplicationController < ActionController::Base\n    end\n  end\nend\n/
-    assert_file "hyphenated-name/app/models/hyphenated/name/application_record.rb",          /module Hyphenated\n  module Name\n    class ApplicationRecord < ActiveRecord::Base\n      self\.abstract_class = true\n    end\n  end\nend/
+    assert_file "hyphenated-name/app/models/hyphenated/name/application_record.rb",          /module Hyphenated\n  module Name\n    class ApplicationRecord < ActiveRecord::Base\n      self.abstract_class = true\n    end\n  end\nend/
     assert_file "hyphenated-name/app/jobs/hyphenated/name/application_job.rb",               /module Hyphenated\n  module Name\n    class ApplicationJob < ActiveJob::Base/
-    assert_file "hyphenated-name/app/mailers/hyphenated/name/application_mailer.rb",         /module Hyphenated\n  module Name\n    class ApplicationMailer < ActionMailer::Base\n      default from: 'from@example\.com'\n      layout 'mailer'\n    end\n  end\nend/
+    assert_file "hyphenated-name/app/mailers/hyphenated/name/application_mailer.rb",         /module Hyphenated\n  module Name\n    class ApplicationMailer < ActionMailer::Base\n      default from: "from@example\.com"\n      layout "mailer"\n    end\n  end\nend/
     assert_file "hyphenated-name/app/helpers/hyphenated/name/application_helper.rb",         /module Hyphenated\n  module Name\n    module ApplicationHelper\n    end\n  end\nend/
     assert_file "hyphenated-name/app/views/layouts/hyphenated/name/application.html.erb" do |contents|
       assert_match "<title>Hyphenated name</title>", contents
@@ -390,14 +432,14 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     assert_file "my_hyphenated-name/app/assets/stylesheets/my_hyphenated/name"
     assert_file "my_hyphenated-name/app/assets/images/my_hyphenated/name"
     assert_file "my_hyphenated-name/config/routes.rb",                                             /MyHyphenated::Name::Engine\.routes\.draw do/
-    assert_file "my_hyphenated-name/lib/my_hyphenated/name/version.rb",                            /module MyHyphenated\n  module Name\n    VERSION = '0\.1\.0'\n  end\nend/
+    assert_file "my_hyphenated-name/lib/my_hyphenated/name/version.rb",                            /module MyHyphenated\n  module Name\n    VERSION = "0\.1\.0"\n  end\nend/
     assert_file "my_hyphenated-name/lib/my_hyphenated/name/engine.rb",                             /module MyHyphenated\n  module Name\n    class Engine < ::Rails::Engine\n      isolate_namespace MyHyphenated::Name\n    end\n  end\nend/
     assert_file "my_hyphenated-name/lib/my_hyphenated/name.rb",                                    /require "my_hyphenated\/name\/engine"/
     assert_file "my_hyphenated-name/test/dummy/config/routes.rb",                                  /mount MyHyphenated::Name::Engine => "\/my_hyphenated-name"/
     assert_file "my_hyphenated-name/app/controllers/my_hyphenated/name/application_controller.rb", /module MyHyphenated\n  module Name\n    class ApplicationController < ActionController::Base\n    end\n  end\nend\n/
-    assert_file "my_hyphenated-name/app/models/my_hyphenated/name/application_record.rb",          /module MyHyphenated\n  module Name\n    class ApplicationRecord < ActiveRecord::Base\n      self\.abstract_class = true\n    end\n  end\nend/
+    assert_file "my_hyphenated-name/app/models/my_hyphenated/name/application_record.rb",          /module MyHyphenated\n  module Name\n    class ApplicationRecord < ActiveRecord::Base\n      self.abstract_class = true\n    end\n  end\nend/
     assert_file "my_hyphenated-name/app/jobs/my_hyphenated/name/application_job.rb",               /module MyHyphenated\n  module Name\n    class ApplicationJob < ActiveJob::Base/
-    assert_file "my_hyphenated-name/app/mailers/my_hyphenated/name/application_mailer.rb",         /module MyHyphenated\n  module Name\n    class ApplicationMailer < ActionMailer::Base\n      default from: 'from@example\.com'\n      layout 'mailer'\n    end\n  end\nend/
+    assert_file "my_hyphenated-name/app/mailers/my_hyphenated/name/application_mailer.rb",         /module MyHyphenated\n  module Name\n    class ApplicationMailer < ActionMailer::Base\n      default from: "from@example\.com"\n      layout "mailer"\n    end\n  end\nend/
     assert_file "my_hyphenated-name/app/helpers/my_hyphenated/name/application_helper.rb",         /module MyHyphenated\n  module Name\n    module ApplicationHelper\n    end\n  end\nend/
     assert_file "my_hyphenated-name/app/views/layouts/my_hyphenated/name/application.html.erb" do |contents|
       assert_match "<title>My hyphenated name</title>", contents
@@ -412,14 +454,14 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     assert_file "deep-hyphenated-name/app/assets/stylesheets/deep/hyphenated/name"
     assert_file "deep-hyphenated-name/app/assets/images/deep/hyphenated/name"
     assert_file "deep-hyphenated-name/config/routes.rb",                                               /Deep::Hyphenated::Name::Engine\.routes\.draw do/
-    assert_file "deep-hyphenated-name/lib/deep/hyphenated/name/version.rb",                            /module Deep\n  module Hyphenated\n    module Name\n      VERSION = '0\.1\.0'\n    end\n  end\nend/
+    assert_file "deep-hyphenated-name/lib/deep/hyphenated/name/version.rb",                            /module Deep\n  module Hyphenated\n    module Name\n      VERSION = "0\.1\.0"\n    end\n  end\nend/
     assert_file "deep-hyphenated-name/lib/deep/hyphenated/name/engine.rb",                             /module Deep\n  module Hyphenated\n    module Name\n      class Engine < ::Rails::Engine\n        isolate_namespace Deep::Hyphenated::Name\n      end\n    end\n  end\nend/
     assert_file "deep-hyphenated-name/lib/deep/hyphenated/name.rb",                                    /require "deep\/hyphenated\/name\/engine"/
     assert_file "deep-hyphenated-name/test/dummy/config/routes.rb",                                    /mount Deep::Hyphenated::Name::Engine => "\/deep-hyphenated-name"/
     assert_file "deep-hyphenated-name/app/controllers/deep/hyphenated/name/application_controller.rb", /module Deep\n  module Hyphenated\n    module Name\n      class ApplicationController < ActionController::Base\n      end\n    end\n  end\nend\n/
-    assert_file "deep-hyphenated-name/app/models/deep/hyphenated/name/application_record.rb",          /module Deep\n  module Hyphenated\n    module Name\n      class ApplicationRecord < ActiveRecord::Base\n        self\.abstract_class = true\n      end\n    end\n  end\nend/
+    assert_file "deep-hyphenated-name/app/models/deep/hyphenated/name/application_record.rb",          /module Deep\n  module Hyphenated\n    module Name\n      class ApplicationRecord < ActiveRecord::Base\n        self.abstract_class = true\n      end\n    end\n  end\nend/
     assert_file "deep-hyphenated-name/app/jobs/deep/hyphenated/name/application_job.rb",               /module Deep\n  module Hyphenated\n    module Name\n      class ApplicationJob < ActiveJob::Base/
-    assert_file "deep-hyphenated-name/app/mailers/deep/hyphenated/name/application_mailer.rb",         /module Deep\n  module Hyphenated\n    module Name\n      class ApplicationMailer < ActionMailer::Base\n        default from: 'from@example\.com'\n        layout 'mailer'\n      end\n    end\n  end\nend/
+    assert_file "deep-hyphenated-name/app/mailers/deep/hyphenated/name/application_mailer.rb",         /module Deep\n  module Hyphenated\n    module Name\n      class ApplicationMailer < ActionMailer::Base\n        default from: "from@example\.com"\n        layout "mailer"\n      end\n    end\n  end\nend/
     assert_file "deep-hyphenated-name/app/helpers/deep/hyphenated/name/application_helper.rb",         /module Deep\n  module Hyphenated\n    module Name\n      module ApplicationHelper\n      end\n    end\n  end\nend/
     assert_file "deep-hyphenated-name/app/views/layouts/deep/hyphenated/name/application.html.erb" do |contents|
       assert_match "<title>Deep hyphenated name</title>", contents
@@ -428,18 +470,79 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     end
   end
 
+  def test_creating_full_engine_mode_adds_keeps
+    run_generator [destination_root, "--full"]
+    folders_with_keep = %w(
+      app/models/concerns
+      app/controllers/concerns
+      test/fixtures/files
+      test/controllers
+      test/mailers
+      test/models
+      test/helpers
+      test/integration
+    )
+    folders_with_keep.each do |folder|
+      assert_file("#{folder}/.keep")
+    end
+  end
+
+  def test_creating_full_api_engine_adds_keeps
+    run_generator [destination_root, "--full", "--api"]
+    folders_with_keep = %w(
+      app/models/concerns
+      app/controllers/concerns
+      test/fixtures/files
+      test/controllers
+      test/mailers
+      test/models
+      test/integration
+    )
+    folders_with_keep.each do |folder|
+      assert_file("#{folder}/.keep")
+    end
+    assert_no_file("test/helpers/.keep")
+  end
+
+  def test_creating_mountable_engine_mode_adds_keeps
+    run_generator [destination_root, "--mountable"]
+    folders_with_keep = %w(
+      app/models/concerns
+      app/controllers/concerns
+      test/fixtures/files
+      test/controllers
+      test/mailers
+      test/models
+      test/helpers
+      test/integration
+    )
+    folders_with_keep.each do |folder|
+      assert_file("#{folder}/.keep")
+    end
+  end
+
   def test_creating_gemspec
     run_generator
     assert_file "bukkits.gemspec", /spec\.name\s+= "bukkits"/
-    assert_file "bukkits.gemspec", /spec\.files = Dir\["\{app,config,db,lib\}\/\*\*\/\*", "MIT-LICENSE", "Rakefile", "README\.md"\]/
+    assert_file "bukkits.gemspec", /spec\.files = Dir.chdir\(File\.expand_path\(__dir__\)\)\s+do\s+Dir\["\{app,config,db,lib\}\/\*\*\/\*", "MIT-LICENSE", "Rakefile", "README\.md"\]\s+end/
     assert_file "bukkits.gemspec", /spec\.version\s+ = Bukkits::VERSION/
   end
 
+  def test_gemspec_uses_optimistic_rails_version_constraint
+    rails_version = "1.2.3.4.pre5"
+
+    Rails.stub(:gem_version, Gem::Version.new(rails_version)) do
+      run_generator
+    end
+
+    assert_file "bukkits.gemspec", /add_dependency "rails", ">= #{Regexp.escape rails_version}"/
+  end
+
   def test_usage_of_engine_commands
     run_generator [destination_root, "--full"]
-    assert_file "bin/rails", /ENGINE_PATH = File\.expand_path\('\.\.\/lib\/bukkits\/engine', __dir__\)/
-    assert_file "bin/rails", /ENGINE_ROOT = File\.expand_path\('\.\.', __dir__\)/
-    assert_file "bin/rails", %r|APP_PATH = File\.expand_path\('\.\./test/dummy/config/application', __dir__\)|
+    assert_file "bin/rails", /ENGINE_PATH = File\.expand_path\("\.\.\/lib\/bukkits\/engine", __dir__\)/
+    assert_file "bin/rails", /ENGINE_ROOT = File\.expand_path\("\.\.", __dir__\)/
+    assert_file "bin/rails", %r|APP_PATH = File\.expand_path\("\.\./test/dummy/config/application", __dir__\)|
     assert_file "bin/rails", /require "rails\/all"/
     assert_file "bin/rails", /require "rails\/engine\/commands"/
   end
@@ -481,10 +584,42 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     end
   end
 
-  def test_dummy_application_loads_plugin
+  def test_plugin_passes_generated_test
+    run_generator
+    prepare_plugin(destination_root)
+
+    in_plugin_context(destination_root) do
+      output = `bin/test 2>&1`
+      assert_predicate $?, :success?, "Command failed: #{output}"
+    end
+  end
+
+  def test_dummy_application_sets_include_all_helpers_to_false_for_mountable
+    run_generator [destination_root, "--mountable"]
+
+    assert_file "test/dummy/config/application.rb", /^    config\.action_controller\.include_all_helpers = false$/
+  end
+
+  def test_dummy_application_sets_include_all_helpers_to_false_for_full
+    run_generator [destination_root, "--full"]
+
+    assert_file "test/dummy/config/application.rb", /include_all_helpers/
+  end
+
+  def test_dummy_application_does_not_set_include_all_helpers_for_regular
     run_generator
 
-    assert_file "test/dummy/config/application.rb", /^require "bukkits"/
+    assert_file "test/dummy/config/application.rb" do |content|
+      assert_no_match(/include_all_helpers/, content)
+    end
+  end
+
+  def test_dummy_application_does_not_set_include_all_helpers_for_api
+    run_generator [destination_root, "--mountable", "--api"]
+
+    assert_file "test/dummy/config/application.rb" do |content|
+      assert_no_match(/include_all_helpers/, content)
+    end
   end
 
   def test_dummy_application_uses_dynamic_rails_version_number
@@ -495,14 +630,6 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     end
   end
 
-  def test_dummy_application_skip_listen_by_default
-    run_generator
-
-    assert_file "test/dummy/config/environments/development.rb" do |contents|
-      assert_match(/^\s*# config\.file_watcher = ActiveSupport::EventedFileUpdateChecker/, contents)
-    end
-  end
-
   def test_ensure_that_gitignore_can_be_generated_from_a_template_for_dummy_path
     FileUtils.cd(Rails.root)
     run_generator([destination_root, "--dummy_path", "spec/dummy", "--skip-test"])
@@ -511,30 +638,6 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     end
   end
 
-  def test_unnecessary_files_are_not_generated_in_dummy_application
-    run_generator
-    assert_no_file "test/dummy/.gitignore"
-    assert_no_file "test/dummy/.ruby-version"
-    assert_no_file "test/dummy/db/seeds.rb"
-    assert_no_file "test/dummy/Gemfile"
-    assert_no_file "test/dummy/public/robots.txt"
-    assert_no_file "test/dummy/README.md"
-    assert_no_file "test/dummy/config/master.key"
-    assert_no_file "test/dummy/config/credentials.yml.enc"
-    assert_no_directory "test/dummy/lib/tasks"
-    assert_no_directory "test/dummy/test"
-    assert_no_directory "test/dummy/vendor"
-    assert_no_directory "test/dummy/.git"
-  end
-
-  def test_spring_binstub_is_not_generated_in_dummy_application
-    run_generator
-    assert_no_file "test/dummy/bin/spring"
-    assert_file "test/dummy/bin/rails" do |contents|
-      assert_no_match %r/spring/, contents
-    end
-  end
-
   def test_skipping_test_files
     run_generator [destination_root, "--skip-test"]
     assert_no_directory "test"
@@ -543,12 +646,64 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     end
   end
 
+  def test_dummy_application_skips_asset_pipeline_when_simple_railtie
+    run_generator
+
+    assert_no_gem "sprockets-rails"
+    assert_no_file "test/dummy/config/initializers/assets.rb"
+    assert_file "test/dummy/config/environments/development.rb" do |content|
+      assert_no_match "config.assets", content
+    end
+  end
+
+  def test_dummy_application_configures_asset_pipeline_when_mountable
+    run_generator [destination_root, "--mountable"]
+
+    assert_gem "sprockets-rails"
+    assert_file "test/dummy/app/assets/config/manifest.js"
+  end
+
+  def test_dummy_application_configures_asset_pipeline_when_full
+    run_generator [destination_root, "--full"]
+
+    assert_gem "sprockets-rails"
+    assert_file "test/dummy/app/assets/config/manifest.js"
+  end
+
+  def test_dummy_application_skips_asset_pipeline_when_flag_skip_asset_pipeline
+    run_generator [destination_root, "--mountable", "--skip-asset-pipeline"]
+
+    assert_no_gem "sprockets-rails"
+    assert_no_file "test/dummy/config/initializers/assets.rb"
+    assert_file "test/dummy/config/environments/development.rb" do |content|
+      assert_no_match "config.assets", content
+    end
+  end
+
+  def test_dummy_application_respects_asset_pipeline_gem_choice
+    run_generator [destination_root, "--mountable", "--asset-pipeline=propshaft"]
+
+    assert_gem "propshaft"
+    assert_no_gem "sprockets-rails"
+    assert_file "test/dummy/config/initializers/assets.rb"
+    assert_file "test/dummy/config/environments/development.rb" do |content|
+      assert_no_match "config.assets", content
+    end
+  end
+
+  def test_no_asset_pipeline_gem_when_no_dummy_application
+    run_generator [destination_root, "--mountable", "--skip-test"]
+
+    assert_no_gem "sprockets-rails"
+    assert_no_directory "test/dummy"
+  end
+
   def test_skipping_gemspec
     run_generator [destination_root, "--skip-gemspec"]
     assert_no_file "bukkits.gemspec"
     assert_file "Gemfile" do |contents|
       assert_no_match("gemspec", contents)
-      assert_match(/gem 'rails'/, contents)
+      assert_match(/gem "rails"/, contents)
     end
   end
 
@@ -557,54 +712,49 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     assert_no_file "bukkits.gemspec"
     assert_file "Gemfile" do |contents|
       assert_no_match("gemspec", contents)
-      assert_match(/gem 'rails'/, contents)
+      assert_match(/gem "rails"/, contents)
     end
   end
 
   def test_creating_plugin_in_app_directory_adds_gemfile_entry
-    # simulate application existence
-    gemfile_path = "#{Rails.root}/Gemfile"
-    Object.const_set("APP_PATH", Rails.root)
-    FileUtils.touch gemfile_path
-    File.write(gemfile_path, "#foo")
+    with_simulated_app do |gemfile_path|
+      File.write(gemfile_path, "#foo")
 
-    run_generator
+      run_generator
 
-    assert_file gemfile_path, /^gem 'bukkits', path: 'tmp\/bukkits'/
-  ensure
-    Object.send(:remove_const, "APP_PATH")
-    FileUtils.rm gemfile_path
+      assert_file gemfile_path, /^gem "bukkits", path: "tmp\/bukkits"/
+    end
   end
 
   def test_creating_plugin_only_specify_plugin_name_in_app_directory_adds_gemfile_entry
-    # simulate application existence
-    gemfile_path = "#{Rails.root}/Gemfile"
-    Object.const_set("APP_PATH", Rails.root)
-    FileUtils.touch gemfile_path
+    with_simulated_app do |gemfile_path|
+      FileUtils.cd(destination_root)
+      run_generator ["bukkits"]
 
-    FileUtils.cd(destination_root)
-    run_generator ["bukkits"]
-
-    assert_file gemfile_path, /gem 'bukkits', path: 'bukkits'/
-  ensure
-    Object.send(:remove_const, "APP_PATH")
-    FileUtils.rm gemfile_path
+      assert_file gemfile_path, /gem "bukkits", path: "bukkits"/
+    end
   end
 
   def test_skipping_gemfile_entry
-    # simulate application existence
-    gemfile_path = "#{Rails.root}/Gemfile"
-    Object.const_set("APP_PATH", Rails.root)
-    FileUtils.touch gemfile_path
+    with_simulated_app do |gemfile_path|
+      run_generator [destination_root, "--skip-gemfile-entry"]
 
-    run_generator [destination_root, "--skip-gemfile-entry"]
+      assert_file gemfile_path do |contents|
+        assert_no_match(/gem "bukkits", path: "tmp\/bukkits"/, contents)
+      end
+    end
+  end
 
-    assert_file gemfile_path do |contents|
-      assert_no_match(/gem 'bukkits', path: 'tmp\/bukkits'/, contents)
+  def test_creating_plugin_in_application_skips_license
+    with_simulated_app do |gemfile_path|
+      FileUtils.cd(destination_root)
+      run_generator ["bukkits"]
+
+      assert_file "bukkits/bukkits.gemspec" do |contents|
+        assert_no_match(/spec.license/, contents)
+      end
+      assert_no_file "bukkits/MIT-LICENSE"
     end
-  ensure
-    Object.send(:remove_const, "APP_PATH")
-    FileUtils.rm gemfile_path
   end
 
   def test_generating_controller_inside_mountable_engine
@@ -640,6 +790,15 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     end
   end
 
+  def test_no_year_in_license_file
+    year = Date.today.year
+
+    run_generator
+    assert_file "MIT-LICENSE" do |contents|
+      assert_no_match(/#{year}/, contents)
+    end
+  end
+
   def test_no_details_from_git_when_skip_git
     name = "TODO: Write your name"
     email = "TODO: Write your email address"
@@ -700,7 +859,7 @@ class PluginGeneratorTest < Rails::Generators::TestCase
 
     assert_file "app/models/bukkits/article.rb"
     assert_file "app/controllers/bukkits/articles_controller.rb" do |content|
-      assert_match "only: [:show, :update, :destroy]", content
+      assert_match "only: %i[ show update destroy ]", content
     end
 
     assert_no_directory "app/assets"
@@ -767,12 +926,27 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     quietly { Rails::Engine::Updater.run(:create_bin_files) }
 
     assert_file "#{destination_root}/bin/rails" do |content|
-      assert_match(%r|APP_PATH = File\.expand_path\('\.\./test/dummy/config/application', __dir__\)|, content)
+      assert_match(%r|APP_PATH = File\.expand_path\("\.\./test/dummy/config/application", __dir__\)|, content)
     end
   ensure
     Object.send(:remove_const, "ENGINE_ROOT")
   end
 
+  def test_railtie_test_command
+    run_generator [destination_root]
+    assert_file ".github/workflows/ci.yml", /run: bin\/test/
+  end
+
+  def test_engine_test_command
+    run_generator [destination_root, "--full"]
+    assert_file ".github/workflows/ci.yml", /run: bin\/rails db:test:prepare test/
+  end
+
+  def test_engine_without_active_record_test_command
+    run_generator [destination_root, "--full", "--skip-active-record"]
+    assert_file ".github/workflows/ci.yml", /run: bin\/rails test/
+  end
+
   private
     def action(*args, &block)
       silence(:stdout) { generator.send(*args, &block) }
@@ -782,25 +956,14 @@ class PluginGeneratorTest < Rails::Generators::TestCase
       ::DEFAULT_PLUGIN_FILES
     end
 
-    def assert_match_sqlite3(contents)
-      if defined?(JRUBY_VERSION)
-        assert_match(/group :development do\n  gem 'activerecord-jdbcsqlite3-adapter'\nend/, contents)
-      else
-        assert_match(/group :development do\n  gem 'sqlite3'\nend/, contents)
-      end
-    end
+    def with_simulated_app
+      gemfile_path = "#{Rails.root}/Gemfile"
+      Object.const_set("APP_PATH", Rails.root)
+      FileUtils.touch gemfile_path
 
-    def assert_generates_without_bundler(options = {})
-      generator([destination_root], options)
-
-      command_check = -> command do
-        if command == "install"
-          flunk "`install` expected to not be called."
-        end
-      end
-
-      generator.stub :bundle_command, command_check do
-        quietly { generator.invoke_all }
-      end
+      yield gemfile_path
+    ensure
+      Object.send(:remove_const, "APP_PATH")
+      FileUtils.rm gemfile_path
     end
 end
